/* HTTaP.h
   (C) 2011,2017,2020 Yann Guidon whygee@f-cpu.org
   The source code of this project is released under the AGPLv3 or later,
   see https://www.gnu.org/licenses/agpl-3.0.en.html

version 20170327: split from serv_11_CORS.c, a lot of renaming
version 20170331: more options and some standard error messages
version 20200415: moved the #includes and BSD socket stuff to ygtcpip.h

This file includes everything and lets you configure some parameters.
 */


/*
 Some default values and the names of the
 environment variables that change them :
 */

/* Uncomment (or gcc -D) to enable debug messages:
#define HTTAP_VERBOSE */

/* Uncomment (or gcc -D) to display the IP address of the client
#define SHOW_CLIENT */

/* Uncomment (or gcc -D) to enable the loopback server
#define ENABLE_LOOPBACK   /!\ not yet implemented */

#ifndef HTTAP_SERVER_ID
#define HTTAP_SERVER_ID     "Default_HTTaP"
#endif

#ifndef HTTAP_VERSION
#define HTTAP_VERSION       "20200427"
#endif

#ifndef HTTAP_TCPPORT
#define HTTAP_TCPPORT         "60000"
#endif

#define HTTAP_TCPPORT_NAME    "HTTAP_TCPPORT"

#ifndef HTTAP_KEEPALIVE
#define HTTAP_KEEPALIVE  (15)   /* timeout in seconds */
#endif
#define HTTAP_KEEPALIVE_NAME  "HTTAP_KEEPALIVE"

#ifndef HTTAP_POLL_SLOW_MS
#define HTTAP_POLL_FAST_MS   (50)  // ms
#endif
#ifndef HTTAP_POLL_SLOW_MS
#define HTTAP_POLL_SLOW_MS   (500) // ms
#endif


#ifdef HTTAP_SERVE_FILES
  /* Path to the sandbox to chroot(): */
  #define HTTAP_STATICPATH_NAME "HTTAP_STATICPATH"
  /* Default name of the root page */
  #define HTTAP_ROOTPAGE        "index.html"
  #define HTTAP_ROOTPAGE_NAME   "HTTAP_ROOTPAGE"
  /* New user name */
  #define HTTAP_USER_NAME       "HTTAP_USER"
  /* New group name */
  #define HTTAP_GROUP_NAME      "HTTAP_GROUP"
#endif

#ifndef  ACCESS_CONTROL_ALLOW_ORIGIN
  #ifndef HTTAP_SERVE_FILES
    #define ACCESS_CONTROL_ALLOW_ORIGIN "Access-Control-Allow-Origin: *\x0d\x0a"
  #else
    #define ACCESS_CONTROL_ALLOW_ORIGIN
  #endif
#endif

////////////////////// HTTP stuff //////////////////////

// I don't understand why this creates some weird displays, it must be checked
//#define HTTP_NOCACHE "Cache-Control: no-cache\x0d\x0aExpires: 0\x0d\x0a"

#define GET_HEADER_LEN 5
char GET_header[GET_HEADER_LEN]="GET /";
#define POST_HEADER_LEN 6
char POST_header[POST_HEADER_LEN]="POST /";
#define PING_HEADER_LEN 4
char PING_header[PING_HEADER_LEN]="ping";


char HTTaP_header[]="HTTP/1.1 200 OK\x0d\x0a"
ACCESS_CONTROL_ALLOW_ORIGIN
"Content-Type: application/json\x0d\x0a";

char HTTaP_json_root[]="{\n"
" \"HTTaP_version\":\"" HTTAP_VERSION "\",\n"
#ifdef HTTAP_SERVER_ID
" \"ID\":\"" HTTAP_SERVER_ID "\",\n"
#endif
" \"Services\":\""
#ifdef ENABLE_LOOPBACK
"Loopback "
#endif
#ifdef HTTAP_SERVE_FILES
"Files "
#endif
"\"\n"
"}";


/* A few minimalist error pages: */
char ok200[]="HTTP/1.1 200 OK\x0d\x0a"
"%s" //"Content-Type: %s\x0d\x0a"
ACCESS_CONTROL_ALLOW_ORIGIN
"Content-Length: %d\x0d\x0a\x0d\x0a";

char pong200[]="HTTP/1.1 200 OK\x0d\x0a"
"Content-Type: text/plain\x0d\x0a"
"Cache-Control: no-cache\x0d\x0a"
"Content-Length: 4\x0d\x0a\x0d\x0a"
"PONG";

char err400[]="HTTP/1.1 400 Invalid\x0d\x0a"
"Content-Type: text/html\x0d\x0a"
"Content-Length: 77\x0d\x0a\x0d\x0a"
"<html><body>Bad request<br>"
"<img src=\"https://http.cat/400.jpg\"></body></html>";

char err400key[]="HTTP/1.1 400 Key not found\x0d\x0a"
"Content-Type: text/html\x0d\x0a"
"Content-Length: 61\x0d\x0a\x0d\x0a"
"<html><body>HTTaP key not found.<br>"
"<a href=\"/\">main site</a>";

char err403[]="HTTP/1.1 403 Forbidden\x0d\x0a"
"Content-Type: text/html\x0d\x0a"
"Content-Length: 75\x0d\x0a\x0d\x0a"
"<html><body>Forbidden<br>"
"<img src=\"https://http.cat/403.jpg\"></body></html>";

char err404[]="HTTP/1.1 404 Not Found\x0d\x0a"
"Content-Type: text/html\x0d\x0a"
"Content-Length: 75\x0d\x0a\x0d\x0a"
"<html><body>Not Found<br>"
"<img src=\"https://http.cat/404.jpg\"></body></html>";

char err405[]="HTTP/1.1 405 Method not allowed\x0d\x0a"
"Content-Type: text/html\x0d\x0a"
"Content-Length: 84\x0d\x0a\x0d\x0a"
"<html><body>Method not allowed<br>"
"<img src=\"https://http.cat/405.jpg\"></body></html>";

char err500[]="HTTP/1.1 500 Internal Server Error\x0d\x0a"
"Content-Type: text/html\x0d\x0a"
"Content-Length: 81\x0d\x0a\x0d\x0a"
"<html><body>Internal error<br>"
"<img src=\"https://http.cat/500.jpg\"></body></html>";


#define HTTAP_BUFFER_LEN (65536) /* size of the receive and emit buffer (64 is quite overkill) */
#define HEADER_BUFFER_LEN (4096) /* holds header informations from auxiliary files (MIME) */
#define MAX_URI_LEN       (1024) /* let's be reasonable, such a long filename
 is not common... unless you play with http://www.falstad.com/circuit/circuitjs.html */

#include "src/support.h"
#include "src/manager.h"
#include "src/ygtcpip.h"
#include "src/support.c"
#include "src/ygtcpip.c"
#include "src/HTTaP.c"
#include "src/manager.c"
